function retval = GMAVariableCN(fcn, varargin)
% Coordination number variable for GMA

if nargin < 1
    error('GMA:internal:GMAVariableCN:nargin1', 'There must be at least one argument.')
end

switch fcn
    case 'getVariableName'
        retval = 'Coordination Number';
    case 'getVersion'
        retval = 1.0;
    case 'getHooks'
        retval = {'variables'};
    case 'getDependencies'
        retval = {};
    case 'getDescription'
        retval = 'The number of particles touching each particle.';
    case 'getProperties'
        retval = struct('average', true, 'trackParticle', true, 'singleValue', false, 'neighborList', true);
    case 'calculate'
        if nargin < 2; error('GMA:internal:GMAVariableCN:nargin3', 'There must be three arguments.'); end
        GMAVariableCN_calculate(varargin{1}, varargin{2})
    otherwise
        error('GMA:internal:GMAVariableCN:unrecognizedFcn', 'The function was unrecognized.')
end

    function GMAVariableCN_calculate(dataObj, wh)
        waitbar(0, wh, 'Calculating Coordination Number')
        % It depends on the coordinates.
        if ~dataObj.hasCoordinates
            error('GMA:info:GMAVariableCN:needCoordinates', 'coordinates')
        end
        for n = 1:length(dataObj.timesteps)
            if ~(dataObj.timesteps{n}.radius > 0)
                if ~(dataObj.defaultRadius > 0)
                    error('GMA:info:GMAVariableCN:needRadius', 'radius')
                end
                dataObj.timesteps{n}.setRadius(dataObj.defaultRadius)
            end
            if ~dataObj.timesteps{n}.boundaries.isSet
                if ~dataObj.defaultBoundaries.isSet
                    error('GMA:info:GMAVariableCN:neededBoundaries', 'boundaries')
                end
                dataObj.timesteps{n}.setBoundaries(dataObj.defaultBoundaries)
            end
        end
        waitLength = length(dataObj.timesteps) * dataObj.numRealizations;
        for n = 1:length(dataObj.timesteps)
            time = dataObj.timesteps{n};
            % Initialize the neighbor list and coordination number.
            CNNeighbors = cell(dataObj.numParticles, 1, dataObj.numRealizations);
            CN = zeros(dataObj.numParticles, 1, dataObj.numRealizations);
            cutoff = time.radius * 2.00001;
            for m = 1:dataObj.numRealizations
                waitbar((n*m-0.5)/waitLength, wh, ['Realization ' num2str(m) ' in timestep ' num2str(n) '.'])
                workCoords = time.coordinates(:,:,m);
                % Add a particle index.
                workCoords(:,4) = 1:dataObj.numParticles;
                if time.boundaries.periodicx
                    % The width in the x-direction
                    width = time.boundaries.upperx - time.boundaries.lowerx;
                    % Copy the particles whose x position is less than the cutoff of the
                    % lower box bound plus the copyRatio times the width.
                    % The syntax used here is called logical indexing.
                    addParticles1 = workCoords(workCoords(:,1) < (time.boundaries.lowerx + cutoff),:);
                    % Move the copied particles to ouside the box on the other side.
                    addParticles1(:,1) = addParticles1(:,1) + width;
                    % We cannot immediately add these particles to the list of coordinates
                    % because then the added particles will be added again below and will
                    % overlap some other particles.
                    % Do it with the other side.
                    addParticles2 = workCoords(workCoords(:,1) > (time.boundaries.upperx - cutoff),:);
                    addParticles2(:,1) = addParticles2(:,1) - width;
                    workCoords = vertcat(workCoords, addParticles1, addParticles2); %#ok<AGROW>
                end
                if time.boundaries.periodicy
                    width = time.boundaries.uppery - time.boundaries.lowery;
                    addParticles1 = workCoords(workCoords(:,2) < (time.boundaries.lowery + cutoff),:);
                    addParticles1(:,2) = addParticles1(:,2) + width;
                    addParticles2 = workCoords(workCoords(:,2) > (time.boundaries.uppery - cutoff),:);
                    addParticles2(:,2) = addParticles2(:,2) - width;
                    workCoords = vertcat(workCoords, addParticles1, addParticles2); %#ok<AGROW>
                end
                if time.boundaries.periodicz
                    width = time.boundaries.upperz - time.boundaries.lowerz;
                    addParticles1 = workCoords(workCoords(:,3) < (time.boundaries.lowerz + cutoff),:);
                    addParticles1(:,3) = addParticles1(:,3) + width;
                    addParticles2 = workCoords(workCoords(:,3) > (time.boundaries.upperz - cutoff),:);
                    addParticles2(:,3) = addParticles2(:,3) - width;
                    workCoords = vertcat(workCoords, addParticles1, addParticles2); %#ok<AGROW>
                end
                % If there are particles in both x and y direction, then the edges have
                % already been filled. First, the x-direction was made larger, then
                % the y-direction was made larger, so the edges have been copied along
                % with the rest of the particles. Same for 3 directions.
                clear('addParticles1', 'addParticles2', 'width')
                fullNumParticles = size(workCoords, 1);
                % We only need to check the neighbors of the original
                % particles, not the copied particles.
                for p = 1:dataObj.numParticles
                    for q = p+1:fullNumParticles
                        xDist = abs(workCoords(p,1) - workCoords(q,1));
                        if xDist > cutoff; continue; end
                        yDist = abs(workCoords(p,2) - workCoords(q,2));
                        if yDist > cutoff; continue; end
                        zDist = abs(workCoords(p,3) - workCoords(q,3));
                        if zDist > cutoff; continue; end
                        dist = sqrt(xDist^2 + yDist^2 + zDist^2);
                        if dist > cutoff; continue; end
                        CNNeighbors{workCoords(p,4),1,m}(end+1) = workCoords(q,4);
                        CNNeighbors{workCoords(q,4),1,m}(end+1) = workCoords(p,4);
                        if length(CNNeighbors{workCoords(p,4),1,m}) > 100
                            error('GMA:error:GMAVariableCN:CNlimit', 'Max CN limit of 100 reached.')
                        end
                    end
                end
                
                for p = 1:dataObj.numParticles
                    CNNeighbors{p,1,m} = unique(CNNeighbors{p,1,m});
                    CN(p,1,m) = length(CNNeighbors{p,1,m});
                end
            end
            % Save the variables to the timestep
            dataObj.timesteps{n}.setVariable('Coordination Number', CN)
            dataObj.timesteps{n}.setVariable('CN Neighbor List', CNNeighbors)
        end
        dataObj.variablesSet{end+1} = 'Coordination Number';
        dataObj.variablesSet{end+1} = 'CN Neighbor List';
    end % function GMAVariableCN_calculate
end % function GMAVariableCN


