function retval = GMAVariableVoronoi(fcn, varargin)
% Coordination number variable for GMA

if nargin < 1
    error('GMA:internal:GMAVariableVoronoi:nargin1', 'There must be at least one argument.')
end

switch fcn
    case 'getVariableName'
        retval = 'Voronoi Volume';
    case 'getVersion'
        retval = 1.0;
    case 'getHooks'
        retval = {'variables'};
    case 'getDependencies'
        retval = {};
    case 'getDescription'
        retval = 'The Voronoi volume of each particle.';
    case 'getProperties'
        retval = struct('average', true, 'trackParticle', true, 'singleValue', false, 'neighborList', false);
    case 'calculate'
        if nargin < 2; error('GMA:internal:GMAVariableVoronoi:nargin3', 'There must be three arguments.'); end
        GMAVariableVoronoi_calculate(varargin{1}, varargin{2})
    otherwise
        error('GMA:internal:GMAVariableVoronoi:unrecognizedFcn', 'The function was unrecognized.')
end

    function GMAVariableVoronoi_calculate(dataObj, wh)
        waitbar(0, wh, 'Calculating Voronoi Volume')
        % It depends on the coordinates.
        if ~dataObj.hasCoordinates
            error('GMA:info:GMAVariableVoronoi:needCoordinates', 'coordinates')
        end
        for n = 1:length(dataObj.timesteps)
            if ~dataObj.timesteps{n}.boundaries.isSet
                if ~dataObj.defaultBoundaries.isSet
                    error('GMA:info:GMAVariableVoronoi:neededBoundaries', 'boundaries')
                end
                dataObj.timesteps{n}.setBoundaries(dataObj.defaultBoundaries)
            end
        end
        waitLength = length(dataObj.timesteps) * dataObj.numRealizations;
        cutoffFactor = 0.85;
        for n = 1:length(dataObj.timesteps)
            for m = 1:dataObj.numRealizations
                % Initialize the Voronoi volumes
                vVol = zeros(dataObj.numParticles, 1, dataObj.numRealizations);
                waitbar((n*m-0.5)/waitLength, wh, ['Realization ' num2str(m) ' in timestep ' num2str(n) '.'])
                workCoords = dataObj.timesteps{n}.coordinates(:,:,m);
                if dataObj.timesteps{n}.boundaries.periodicx
                    % The width in the x-direction
                    width = dataObj.timesteps{n}.boundaries.upperx - dataObj.timesteps{n}.boundaries.lowerx;
                    % Copy the particles whose x position is less than the cutoff of the
                    % lower box bound plus the copyRatio times the width.
                    % The syntax used here is called logical indexing.
                    addParticles1 = workCoords(workCoords(:,1) < (dataObj.timesteps{n}.boundaries.lowerx + cutoffFactor * width),:);
                    % Move the copied particles to ouside the box on the other side.
                    addParticles1(:,1) = addParticles1(:,1) + width;
                    % We cannot immediately add these particles to the list of coordinates
                    % because then the added particles will be added again below and will
                    % overlap some other particles.
                    % Do it with the other side.
                    addParticles2 = workCoords(workCoords(:,1) > (dataObj.timesteps{n}.boundaries.upperx - cutoffFactor * width),:);
                    addParticles2(:,1) = addParticles2(:,1) - width;
                    workCoords = vertcat(workCoords, addParticles1, addParticles2); %#ok<AGROW>
                end
                if dataObj.timesteps{n}.boundaries.periodicy
                    width = dataObj.timesteps{n}.boundaries.uppery - dataObj.timesteps{n}.boundaries.lowery;
                    addParticles1 = workCoords(workCoords(:,2) < (dataObj.timesteps{n}.boundaries.lowery + cutoffFactor * width),:);
                    addParticles1(:,2) = addParticles1(:,2) + width;
                    addParticles2 = workCoords(workCoords(:,2) > (dataObj.timesteps{n}.boundaries.uppery - cutoffFactor * width),:);
                    addParticles2(:,2) = addParticles2(:,2) - width;
                    workCoords = vertcat(workCoords, addParticles1, addParticles2); %#ok<AGROW>
                end
                if dataObj.timesteps{n}.boundaries.periodicz
                    width = dataObj.timesteps{n}.boundaries.upperz - dataObj.timesteps{n}.boundaries.lowerz;
                    addParticles1 = workCoords(workCoords(:,3) < (dataObj.timesteps{n}.boundaries.lowerz + cutoffFactor * width),:);
                    addParticles1(:,3) = addParticles1(:,3) + width;
                    addParticles2 = workCoords(workCoords(:,3) > (dataObj.timesteps{n}.boundaries.upperz - cutoffFactor * width),:);
                    addParticles2(:,3) = addParticles2(:,3) - width;
                    workCoords = vertcat(workCoords, addParticles1, addParticles2); %#ok<AGROW>
                end
                % If there are particles in both x and y direction, then the edges have
                % already been filled. First, the x-direction was made larger, then
                % the y-direction was made larger, so the edges have been copied along
                % with the rest of the particles. Same for 3 directions.
                clear('addParticles1', 'addParticles2', 'width')
                % Compute the Delaunay triangulation of the coordinates.
                Triangulation = DelaunayTri(workCoords);
                if ~isequal(Triangulation.X, workCoords)
                    disp('Unequal coordinates with CGAL!')
                end
                % Use the Delaunay triangulation to find the Voronoi cells
                [VerticesLibrary, Indices] = voronoiDiagram(Triangulation);
                % Find all of the Voronoi vertices that are outside the convex hull of
                % particle coordinates.
                % pointLocation will not take infinite locations. We know that the first
                % point in VerticesLibrary is at infinity, and it is the only point at
                % infinity.
                PointArray = pointLocation(Triangulation, VerticesLibrary(2:end,:));
                % Free the memory.
                clear('Triangulation')
                % If I do it now instead of in the for loop, I can save some time and
                % memory. Also, add back the first point, which is clearly outside the
                % convex hull.
                PointArray = logical([1; isnan(PointArray)]);
                
                % Sanity check, in case DelaunayTri discarded some particles that unique
                % didn't catch.
                if length(Indices) ~= size(workCoords, 1)
                    error('MicrostructureAnalyzer:Voronoi:DelaunayTri', ...
                        'DelaunayTri has unexpectedly discarded some particles.')
                end

                % The number of skipped particles must be kept so that the cell volume
                % will be put in the right place.
                % skipped = 0;
                % Preallocate the CellVolume array.
                CellVolume = zeros(dataObj.numParticles, 1);
                
                % This will only go through the real particles, not the fake ones.
                % i is the imaginary number, don't overload it as an integer.
                for p = 1:dataObj.numParticles;
                    % Check if any point is outside the convex hull of the group.
                    % PointArray is 1 if the point is outside and 0 if it is inside. So the
                    % sum is 0 if all points are inside.
                    if any(PointArray(Indices{p,1}))
                        % % The particle should be skipped.
                        % % Remove the associated particle index.
                        % if ParticleIndex
                        %     ParticleIndex(n - skipped) = [];
                        % end
                        % skipped = skipped + 1;
                        % % Shrink the CellVolume array by one because it won't be needed.
                        % CellVolume(end) = [];
                        % This is if you want to keep the particles, but set the values to
                        % an uncomputable value.
                        CellVolume(p) = NaN;
                    else
                        % The number of skipped volumes must be used because p is still
                        % incremented each time.
                        % [trash, CellVolume(p-skipped)] = convexHull(DelaunayTri(VerticesLibrary(Indices{p,1},:)));
                        [trash, CellVolume(p)] = convexHull(DelaunayTri(VerticesLibrary(Indices{p,1},:))); %#ok<ASGLU>
                        clear('trash')
                    end
                end
                
                vVol(:,1,m) = CellVolume;
            end
            % Save the variables to the timestep
            dataObj.timesteps{n}.setVariable('Voronoi Volume', vVol)
        end
        dataObj.variablesSet{end+1} = 'Voronoi Volume';
    end % function GMAVariableVoronoi_calculate
end % function GMAVariableVoronoi


