function retval = GMAVisualizationHistogramGamma(fcn, varargin)

if nargin < 1
    error('GMA:internal:GMAVisualizationHistogramGamma:nargin1', 'There must be at least one argument.')
end

switch fcn
    case 'getVisualizationName'
        retval = 'Histogram with Gamma Fit';
    case 'getVersion'
        retval = 1.0;
    case 'getHooks'
        retval = {'visualizations'};
    case 'getDependencies'
        retval = {};
    case 'getDescription'
        if nargin < 3
            error('GMA:internal:GMAVisualizationHistogramGamma:nargin2', 'There must be two arguments.')
        end
        if checkCompatibility(varargin{2})
            retval = ['Histogram of ' varargin{1} '.'];
        else
            retval = ['The ' varargin{1} ' cannot be displayed with a histogram.'];
        end
    case 'checkCompatibility'
        if nargin < 2
            error('GMA:internal:GMAVisualizationHistogramGamma:nargin2', 'There must be two arguments.')
        end
        retval = checkCompatibility(varargin{1});
    case 'display'
        if nargin ~= 5
            error('GMA:internal:GMAVisualizationHistogramGamma:nargin5', 'There must be five arguments.')
        end
        GMAVisualizationHistogramGamma_display(varargin{1}, varargin{2}, varargin{3}, varargin{4});
    otherwise
        error('GMA:internal:GMAVisualizationHistogramGamma:unrecognizedFcn', 'The function was unrecognized.')
end

    function cRetVal = checkCompatibility(varStruct)
        cRetVal = false;
        if ~varStruct.singleValue
            cRetVal = true;
        end
    end
    function GMAVisualizationHistogramGamma_display(dataObj, displayName, selectedTimestep, props)
        % Somehow, it is not possible to call functions from the
        % GMAFunctions package. Therefore, all variables necessary must be
        % passed.
        if (length(dataObj.times) > 1) && isemtpy(selectedTimestep)
            % If there is more than one timestep, then ask which timestep
            % to do a histogram on.
            error('GMA:info:GMAVisualizationHistogramGamma:singleTimetep', 'singleTimestep')
        end
        % Check to see if the variable has the right properties.
        % Get the display name of the variable.
        if ~checkCompatibility(props)
            error('GMA:error:incompatible', 'incompatible')
        end
        % Assume that the variable exists.
        if length(dataObj.times) > 1
            timeIndex = find(strcmp(dataObj.times, selectedTimestep));
        else
            timeIndex = 1;
        end
        data = dataObj.timesteps{timeIndex}.getVariable(displayName);
        if isempty(data)
            error('GMA:error:GMAVisualizationHistogramGamma:varNotFound', ...
                ['The variable ' displayName ' was not found.'])
        end
        % Take the data from the third dimension (realizations) and make it
        % into one long realization.
        data = reshape(data, [], 1);
        % Remove NaN's
        data(isnan(data)) = [];
        minX = min(data);
        maxX = max(data);
        if minX <= 0
            warnHandle = warndlg(['There are non-positive values in the data, so a gamma ' ...
                'fit cannot be done.'], 'Unable to Gamma Fit');
            set(warnHandle, 'NextPlot', 'new')
            gammaFit = false;
        else
            gammaFit = true;
        end
        % Test to see if it is discrete data or not, and apply a bin width
        % accordingly.
        intdata = int8(data);
        if intdata == data
            binWidth = 1;
        else
            binWidth = (maxX - minX) / floor(sqrt(dataObj.numParticles));
        end
        clear('intdata')
        x = minX:binWidth:maxX;
        histogramData = histc(data, x);
        % This turns it into a percentage.
        histogramData = histogramData ./ dataObj.numParticles * 100;
        if gammaFit
            gamx = x - (binWidth / 2);
            gamx(end + 1) = maxX + (binWidth / 2);
            GammaMLE = gamfit(data);
            GammaData = gampdf(gamx, GammaMLE(1), GammaMLE(2));
            % Normalize the histogram data so that the area under the curve
            % is one, because the area under the gamma fit is one.
            histogramData = histogramData ./ (100 .* binWidth);
        end
        BarLegendString = ['Histogram of ' displayName '\newline average: ' ...
            num2str(mean(data)) '\newline standard deviation: ' num2str(std(data)) ...
            '\newline maximum: ' num2str(maxX) '\newline minimum: ' num2str(minX)];
        clear('data')
        axesHandle = newplot;
        hold(axesHandle, 'on')
        bar(x, histogramData', 1, 'FaceColor', [.9 .5 0], 'EdgeColor', 'none')
        if gammaFit
            plot(gamx, GammaData, 'b')
            legend(axesHandle, {['Normalized ' BarLegendString], ...
                ['Gamma fit with shape ' num2str(GammaMLE(1)) ' and scale ' num2str(GammaMLE(2))]});
            ylabel(axesHandle, 'Normalized Frequency');
        else
            legend(axesHandle, BarLegendString);
            ylabel(axesHandle, 'Percent Frequency')
        end
        xlabel(axesHandle, displayName)
        title(axesHandle, ['Plot of ' displayName '.'])
    end % function GMAVisualizationHistogramGamma_calculate
end % function GMAVisualizationHistogramGamma


