function scr_main(handles, command)
% The main screen, with variables list and visualizations list.

if nargin ~= 2; error('GMA:internal:scr_main:nargin', 'There must be 2 arguments.'); end

par = handles.Container;
% get the list of variables and visualizations
hooks = GMAFunctions.get(handles, 'hooks');
extensionHandles = GMAFunctions.get(handles, 'extensionHandles');
analysisType = GMAFunctions.get(handles, 'analysisType');

switch command
    case 'create'
        createUI()
        resizeUI()
    case 'resize'
        resizeUI()
    case 'nextCallback'
        vars = GMAFunctions.get(handles, 'variableNames');
        selVar = get(handles.varlist, 'Value');
        selViz = get(handles.vizlist, 'Value');
        % Check to see if the variable and visualization are compatible.
        selVar = vars{selVar};
        selViz = hooks.visualizations{selViz};
        varFunc = extensionHandles.(selVar);
        vizFunc = extensionHandles.(selViz);
        if ~vizFunc('checkCompatibility', varFunc('getProperties'))
            desc = vizFunc('getDescription', selVar, varFunc('getProperties'));
            errordlg(desc, 'GMA Error')
            return
        end
        % load the variable
        GMAFunctions.runIt(handles)
    case 'backCallback'
        GMAFunctions.screenManager(handles, 'back')
    case 'listCallback'
        selVar = get(handles.varlist, 'Value');
        vars = GMAFunctions.get(handles, 'variableNames');
        selVar = vars{selVar};
        selViz = get(handles.vizlist, 'Value');
        selViz = hooks.visualizations{selViz};
        updateSelection(selVar, selViz)
    case 'saveCallback'
        [filename pathname] = uiputfile;
        if isnumeric(filename); return; end
        filename = [pathname filename];
        dataObj = GMAFunctions.get(handles, 'dataObj');
        dataObj.serialize(filename)
    otherwise
end

    function createUI()
        uicontrol(par, 'Tag', 'toptext', 'Style', 'text', 'Units', 'pixels', ...
            'String', ['Select a variable from the list and choose a ' ...
            'visualization. Then press "Next."'], 'FontSize', 10)
        varList = cell(size(hooks.variables));
        varHandles = cell(size(hooks.variables));
        for n = 1:length(hooks.variables)
            fcn = extensionHandles.(hooks.variables{n});
            varList{n} = fcn('getVariableName');
            varHandles{n} = fcn;
        end
        GMAFunctions.set(handles, 'variableNames', hooks.variables)
        vizList = cell(size(hooks.visualizations));
        for n = 1:length(hooks.visualizations)
            fcn = extensionHandles.(hooks.visualizations{n});
            vizList{n} = fcn('getVisualizationName');
        end
        uicontrol(par, 'Tag', 'varlist', 'Style', 'listbox', 'Units', 'pixels', ...
            'BackgroundColor', 'white', 'String', varList, 'Callback', ...
            @(h,e)GMAFunctions.scr_main(guihandles(h),'listCallback'), ...
            'Max', 0.5, 'Min', 0, 'Value', 1, 'FontSize', 10)
        uicontrol(par, 'Tag', 'vizlist', 'Style', 'listbox', 'Units', 'pixels', ...
            'BackgroundColor', 'white', 'String', vizList, 'Callback', ...
            @(h,e)GMAFunctions.scr_main(guihandles(h),'listCallback'), ...
            'Max', 0.5, 'Min', 0, 'Value', 1, 'FontSize', 10)
        uicontrol(par, 'Tag', 'vardesc', 'Style', 'text', 'Units', 'pixels', 'FontSize', 10)
        uicontrol(par, 'Tag', 'vizdesc', 'Style', 'text', 'Units', 'pixels', 'FontSize', 10)
        uicontrol(par, 'Tag', 'nextbutton', 'Style', 'pushbutton', 'Units', ...
            'pixels', 'String', 'Next', 'FontSize', 10, 'Callback', ...
            @(h,e)GMAFunctions.scr_main(guihandles(h),'nextCallback'))
        uicontrol(par, 'Tag', 'backbutton', 'Style', 'pushbutton', 'Units', ...
            'pixels', 'String', 'Back', 'FontSize', 10, 'Callback', ...
            @(h,e)GMAFunctions.scr_main(guihandles(h),'backCallback'))
        uicontrol(par, 'Tag', 'savebutton', 'Style', 'pushbutton', 'Units', ...
            'pixels', 'String', 'Save All', 'FontSize', 10, 'Callback', ...
            @(h,e)GMAFunctions.scr_main(guihandles(h),'saveCallback'), ...
            'TooltipString', 'Save all data to a MATLAB file that can be loaded again.')
        % reload the handles
        handles = guihandles(par);
        selectedVar = GMAFunctions.get(handles, 'selectedVariable');
        if isempty(selectedVar)
            selectedVar = hooks.variables{1};
        end
        selectedViz = GMAFunctions.get(handles, 'selectedVisualization');
        if isempty(selectedViz)
            selectedViz = hooks.visualizations{1};
        end
        updateSelection(selectedVar, selectedViz)
    end % function createUI()
    function resizeUI()
        % Get the figure size
        figsize = get(handles.figure, 'Position');
        fWidth = figsize(3);
        fHeight = figsize(4);
        % resize
        set(handles.toptext, 'Position', [10, fHeight-50, fWidth-19, 40])
        set(handles.varlist, 'Position', [10, 145, fWidth/2-14, fHeight-205])
        set(handles.vizlist, 'Position', [fWidth/2+5, 145, fWidth/2-14, fHeight-205])
        set(handles.vardesc, 'Position', [10, 45, fWidth/2-14, 80])
        set(handles.vizdesc, 'Position', [fWidth/2+5, 45, fWidth/2-14, 80])
        set(handles.nextbutton, 'Position', [fWidth-109, 10, 100, 25])
        set(handles.backbutton, 'Position', [10, 10, 100, 25])
        set(handles.savebutton, 'Position', [fWidth/2-60, 10, 120, 25])
    end % function resizeUI
    function updateSelection(selectedVar, selectedViz)
        if nargin < 2 || isempty(selectedVar) || isempty(selectedViz)
            error('GMA:internal:scr_main:updateSelection:nargin', 'There must be 2 arguments.')
        end
        varFcn = extensionHandles.(selectedVar);
        set(handles.vardesc, 'String', varFcn('getDescription'))
        set(handles.varlist, 'Value', find(strcmp(selectedVar, hooks.variables)))
        GMAFunctions.set(handles, 'selectedVariable', selectedVar)
        varDisplay = varFcn('getVariableName');
        vizFcn = extensionHandles.(selectedViz);
        varProps = varFcn('getProperties');
        set(handles.vizdesc, 'String', vizFcn('getDescription', varDisplay, varProps))
        set(handles.vizlist, 'Value', find(strcmp(selectedViz, hooks.visualizations)))
        GMAFunctions.set(handles, 'selectedVisualization', selectedViz)
    end % function updateSelection
end % function
