classdef GMAData < handle
    properties (Constant)
        version = 1.0;
    end % properties (Constant)
    properties (SetAccess = protected)
        isLoaded = false;
        hasCoordinates = false;
        numParticles = 0;
        numRealizations = 0;
        times = {};
        defaultBoundaries = GMABoundary;
        defaultRadius = 0;
    end
    properties
        % The variables that are in the GMATimestep objects.
        variablesSet = {};
        % The entries are GMATimestep objects.
        timesteps = {};
    end
    methods
        function obj = GMAData(filename)
            obj.addFile(filename);
        end % function GMAData
        function addFile(obj, filename)
            % FIXME: this assumes that the files are plain text with three
            % columns, xyz. It also assumes that they are all realizations.
            coords = load(filename{1});
            if isstruct(coords) && isfield(coords, 'obj') && strcmp(class(coords.obj), 'GMAData')
                obj.isLoaded = coords.obj.isLoaded;
                obj.hasCoordinates = coords.obj.hasCoordinates;
                obj.numParticles = coords.obj.numParticles;
                obj.numRealizations = coords.obj.numRealizations;
                obj.times = coords.obj.times;
                obj.defaultBoundaries = coords.obj.defaultBoundaries;
                obj.defaultRadius = coords.obj.defaultRadius;
                obj.variablesSet = coords.obj.variablesSet;
                obj.timesteps = coords.obj.timesteps;
                return
            end
            numCols = size(coords, 2);
            if numCols < 3
                error('GMA:error:GMAData:addFile:invalidtext', ['There are too few columns in ' ...
                    filename{1} '. Is it really a coordinates file?'])
            end
            if isempty(obj.timesteps)
                obj.times{1} = 0;
                obj.timesteps{1} = GMATimestep;
                obj.timesteps{1}.setDimension(3);
            end
            numparticles = size(coords, 1);
            if numCols > 3
                if numCols == 4
                    % The first column is an index.
                    indices = coords(:,1);
                    coords(:,1) = [];
                    if length(unique(indices)) ~= numparticles
                        error('GMA:error:GMAData:addFile:duplicateindex', ...
                            ['There are four columns in the file. The first is assumed to be '...
                            'an index, but it is non-unique. Please preprocess ' filename{1}])
                    end
                    if (max(indices) ~= numparticles) || (min(indices) ~= 0)
                        error('GMA:error:GMAData:addFile:invalidindex', ...
                            ['There are four columns in the file. The first is assumed to be '...
                            'an index, which must go from 1 to the number of particles. ' ...
                            'Please preprocess ' filename{1}])
                    end
                    coords(indices,:) = coords;
                    obj.timesteps{1}.setIndicesAreReal(true)
                else
                    coords(:,4:end) = [];
                end
            end
            obj.numParticles = numparticles;
            obj.numRealizations = length(filename);
            obj.timesteps{1}.coordinates = zeros(obj.numParticles, 3, obj.numRealizations);
            obj.timesteps{1}.coordinates(:,:,1) = coords;
            if obj.numRealizations > 1
                for n = 2:obj.numRealizations
                    coords = load(filename{n});
                    if size(coords, 1) ~= obj.numParticles;
                        error('GMA:error:GMAData:addFile:sameNumParticles', ['There must ' ...
                            'be the same number of particles in all data files. The file ' ...
                            filename{n} ' has ' num2str(size(coords, 1)), ' particles ' ...
                            'instead of ' num2str(obj.numParticles) ' particles in ' filename{1}])
                    end
                    numcols = size(coords, 2);
                    if obj.timesteps{1}.indicesAreReal
                        if numcols ~= 4
                            error('GMA:error:GMAData:addFile:allindices', ...
                                ['Either all files have an index, or no files have an index. ' ...
                                'The file ' filename{n} ' has ' num2str(numcols) ' columns. ' ...
                                ' An index is the first column in a four-column file.'])
                        end
                        indices = coords(:,1);
                        coords(:,1) = [];
                        if length(unique(indices)) ~= obj.numParticles
                            error('GMA:error:GMAData:addFile:duplicateIndex', ['There must ' ...
                            'be the same number of particles in all data files. The file ' ...
                            filename{n} ' has ' num2str(size(coords, 1)), ' particles ' ...
                            'instead of ' num2str(obj.numParticles) ' particles in ' filename{n}])
                        end
                        if (max(indices) ~= obj.numParticles) || (min(indices) ~= 1)
                            error('GMA:error:GMAData:addFile:invalidindex', ...
                                ['There are four columns in the file. The first is assumed to be '...
                                'an index, which must go from 1 to the number of particles. ' ...
                                'Please preprocess ' filename{n}])
                        end
                        coords(indices,:) = coords;
                    elseif numcols < 3
                        error('GMA:error:GMAData:addFile:toofewcols', ['The file ' ...
                            filename{n} ' must have three columns for x, y, and z coordinates.'])
                    elseif numcols > 3
                        coords(:,4:end) = [];
                    end
                    obj.timesteps{1}.coordinates(:,:,n) = coords;
                end
            end
            obj.hasCoordinates = true;
            obj.isLoaded = true;
        end % function addFile
        function answer = empty(obj)
            if isempty(obj.timesteps)
                answer = false;
            else
                answer = true;
            end
        end % function empty
        function setRadius(obj, radius)
            if ~isnumeric(radius) || ~isreal(radius) || (radius <= 0)
                error('GMA:error:GMAData:setRadius:invalid', 'Invalid radius.')
            end
            obj.defaultRadius = radius;
            if isempty(obj.timesteps); return; end
            for n = 1:length(obj.timesteps)
                if ~(obj.timesteps{n}.radius > 0)
                    obj.timesteps{n}.setRadius(radius)
                end
            end
        end % function setRadius
        function setBoundaries(obj, boundsObj)
            % Sets the boundaries
            obj.defaultBoundaries = boundsObj;
            if isempty(obj.timesteps); return; end
            for n = 1:length(obj.timesteps)
                if ~obj.timesteps{n}.boundaries.isSet
                    obj.timesteps{n}.setBoundaries(boundsObj)
                end
            end
        end % function setBoundaries
        function retval = getVariable(obj, varName)
            % Gets a variable from all of the timesteps and returns it as a
            % cell array of values, one entry for each timestep.
            if length(obj.timesteps) < 1
                retval = {};
                return
            end
            retval = cell(size(obj.times));
            for n = 1:length(obj.timesteps)
                retval{n} = obj.timesteps{n}.getVariable(varName);
            end
        end % function getVariable
        function serialize(obj, fileName, varName)
            % Writes the object to a file.
            % If a varName is given, it will write only the coordinates,
            % radius, boundaries, and that variable to a file.
            % TODO: be able to write several timesteps to one file.
            if (nargin < 3) || isempty(varName)
                save(fileName, 'obj')
            else
                data = obj.getVariable(varName);
                % Get the first timestep. Right now, GMA only handles one
                % timestep anyway (see addfile).
                data = data{1};
                % Transform any realizations to one long realization.
                data = reshape(data, [], 1);
                if iscell(data)
                    fHandle = fopen(fileName, 'w');
                    for n = 1:length(data)
                        str = sprintf('%u\t', data{n});
                        str = str(1:end-1);
                        fprintf(fHandle, '%s\n', str);
                    end
                    fclose(fHandle);
                else
                    save(fileName, 'data', '-ascii')
                end
            end
        end % function serialize
        function close(obj)
            obj.timesteps = {};
            obj.times = {};
            obj.isLoaded = false;
            obj.hasCoordinates = false;
            obj.numParticles = 0;
            obj.numRealizations = 0;
            obj.defaultBoundaries = GMABoundary;
            obj.defaultRadius = 0;
            obj.variablesSet = {};
        end
    end % methods
    methods (Static)
        function obj = unserialize(filename)
            % Reads the object data from a file.
            obj = load(filename);
        end % function unserialize
    end % methods (Static)
end % classdef GMAData

