classdef GMATimestep < handle
    % Represents a single timestep, maybe with multiple realizations per
    % timestep.
    %
    % Changelog:
    % 2011-05-14: Nathaniel Ching: Initial structure
    
    properties (Constant)
        version = 1.0;
    end
    properties (SetAccess = protected, GetAccess = public)
        time = 0;
        % dimension is 2 or 3.
        dimension = NaN;
        % indicesAreReal is true if the row number in the coordinates
        % matrix actually represents the particle number, and false if they
        % are jumbled up. If they are jumbled up, then only averages are
        % possible, not particle tracking.
        indicesAreReal = false;
        variableNames = {};
        variables = {};
        radius = NaN;
        boundaries = GMABoundary;
    end % properties (SetAccess = protected, GetAccess = public)
    properties
        % The coordinates are dimensioned as follows:
        %    rows (1st dimension): particle number
        %    columns (2nd dimension): x, y, z
        %    3rd dimension: realizations
        coordinates = [];
    end
    methods
        function setDimension(obj, inputVal)
            % This could be shortened, but this is a way of forcing it to
            % be either 2 or 3, without any shades.
            if inputVal == 2
                obj.dimension = 2;
            elseif inputVal == 3
                obj.dimension = 3;
            else
                throw(MException('GMATimestep:warning', 'The dimension must be 2 or 3.'))
            end
        end % function setDimension
        function setTime(obj, inputVal)
            if ischar(inputVal)
                inputVal = str2double(inputVal);
                if isnan(inputVal)
                    throw(MException('GMATimestep:warning', 'The time must be a number.'))
                end
            elseif ~isnumeric(inputVal)
                throw(MException('GMATimestep:fatal', ...
                    'The input value to GMATimestep.setTime must be a number or a string.'))
            end
            if ~isreal(inputVal)
                inputVal = real(inputVal);
            end
            if inputVal < 0
                throw(MException('GMATimestep:warning', 'The time must be non-negative.'))
            end
            obj.time = double(inputVal);
        end % function setTime
        function setIndicesAreReal(obj, inputVal)
            if inputVal
                obj.indicesAreReal = true;
            else
                obj.indicesAreReal = false;
            end
        end % function setIndicesAreReal
        function setCoordinates(obj, inputCoords, particleIndices, realizationIndices)
            % If particleIndices or realizationIndices are [], '', or ':',
            % then it will cover all indices.
            error(nargchk(2,4,nargin))
            inSize = size(inputCoords);
            if inSize(2) > 3 || inSize(2) < 2
                throw(MException('GMATimestep:fatal', ['The coordinates given have a dimension of ' ...
                    num2str(inSize(2)) '.']))
            end
            if inSize(2) == 2
                inputCoords(:,3,:) = 0;
            end
            if nargin > 2
                if isempty(particleIndices) || strcmp(particleIndices, ':')
                    particleIndices = NaN;
                elseif length(particleIndices) ~= inSize(1)
                    throw(MException('GMATimestep:fatal', ['There are ' num2str(length(particleIndices)) ...
                        ' particle indices given and ' num2str(inSize(1)) ' particles given.']))
                end
            else
                particleIndices = NaN;
            end
            if nargin > 3 && ~isempty(realizationIndices)
                if isempty(realizationIndices) || strcmp(realizationIndices, ':')
                    realizationIndices = NaN;
                elseif length(realizationIndices) ~= inSize(3)
                    throw(MException('GMATimestep:fatal', ['There are ' num2str(length(realizationIndices)) ...
                        ' realization indices given and ' num2str(inSize(3)) ' realizations given.']))
                end
            else
                realizationIndices = NaN;
            end
            if isnan(particleIndices)
                if isnan(realizationIndices)
                    obj.coordinates = inputCoords;
                else
                    obj.coordinates(:,:,realizationIndices) = inputCoords;
                end
            else
                if isnan(realizationIndices)
                    obj.coordinates(particleIndices,:,:) = inputCoords;
                else
                    obj.coordinates(particleIndices,:,realizationIndices) = inputCoords;
                end
            end
        end % function setCoordinates
        function setBoundaries(obj, boundObj)
            obj.boundaries = boundObj;
        end % function setBoundaries
        function setVariable(obj, varName, value)
            obj.variableNames{end+1} = varName;
            obj.variables{end+1} = value;
        end % function setVariable
        function value = getVariable(obj, varName)
            if ~any(strcmp(varName, obj.variableNames)); value = []; return; end
            value = obj.variables{strcmp(varName, obj.variableNames)};
        end % function getVariable
        function setRadius(obj, val)
            if isnumeric(val) && isreal(val) && (val > 0)
                obj.radius = val;
            else
                error('GMA:error:GMATimestep:setRadius:invalid', ...
                    'The radius must be a real, positive number.')
            end
        end % function setRadius
    end % methods
end % classdef GMATimestep
