function gma()
% Granular Microstructure Analyzer
% Analyzes the microstructure of monodisperse granular systems.
%
% Copyright 2009-2011 Granular Science Laboratory, NJIT. All rights
% reserved.
%
% Until I come up with a better license, don't use this software. You've
% been warned!!!
%
% Changelog:
% 2011-05-30: Nathaniel Ching: Original
% 2011-09-13: Nathaniel Ching: Total rewrite

% NOTE: All version checks are done once, in this file. Don't do them
% again, that would be a waste of resources.

% MATLAB version check
if ~exist('verLessThan') %#ok<EXIST> It's not a builtin, but it is a function.
    error('Granular Microstructure Analyzer uses MATLAB 7.9 (R2009b) or higher.')
elseif verLessThan('matlab', '7.9')
    errordlg('Granular Microstructure Analyzer uses MATLAB 7.9 (R2009b) or higher.')
    return
end

GMAVersion = 1.0;

% Load an intro screen here, for user feedback while the rest is
% being done.
waithandle = waitbar(0, {'{\fontsize{16} Granular Microstructure Analyzer}', 'Loading components'});

% Check if the functions package is there. Assume that all functions in the
% package exist.
if ~exist('+GMAFunctions', 'dir')
    close(waithandle)
    errordlg('The folder +GMAFunctions is missing.')
    return
end

waitbar(0.03, waithandle)

% Check if the GMAData, GMATimestep, and GMABoundaries classes have the
% right version.
if ~exist('GMAData', 'class')
    close(waithandle)
    errordlg('The class GMAData does not exist.')
    return
end
if GMAData.version ~= GMAVersion
    close(waithandle)
    errordlg(['The GMAData class is version ' num2str(GMAData.version) ...
        '. The current version is ' num2str(GMAVersion) '.'])
    return
end
waitbar(0.6, waithandle)
if ~exist('GMATimestep', 'class')
    close(waithandle)
    errordlg('The class GMATimestep does not exist.')
    return
end
if GMATimestep.version ~= GMAVersion
    close(waithandle)
    errordlg(['The GMATimestep class is version ' num2str(GMATimestep.version) ...
        '. The current version is ' num2str(GMAVersion) '.'])
    return
end
waitbar(0.09, waithandle)
if ~exist('GMABoundary', 'class')
    close(waithandle)
    errordlg('The class GMABoundaries does not exist.')
    return
end
if GMABoundary.version ~= GMAVersion
    close(waithandle)
    errordlg(['The GMABoundaries class is version ' num2str(GMABoundaries.version) ...
        '. The current version is ' num2str(GMAVersion) '.'])
    return
end

% Check if the extensions package with extension superclass is there.
if ~exist('+GMAExtensions', 'dir')
    close(waithandle)
    errordlg('The folder +GMAExtensions is missing.')
    return
end
waitbar(0.12, waithandle)

% Get list of extensions. Also get the extensions' hooks and dependencies.
% Check the versions.
hooks = struct('variables', {{}}, 'visualizations', {{}});
dependencies = {};
extensionNames = {};
extensionHandles = struct;
tempNames = what('+GMAExtensions');
tempNames = tempNames.m;
[~, tempNames] = cellfun(@fileparts, tempNames, 'UniformOutput', false);
waitIncrement = 0.33/length(tempNames);
waitPosition = 0.12;
for n = 1:length(tempNames)
    name = tempNames{n};
    handle = str2func(['GMAExtensions.' name]);
    skip = false;
    if handle('getVersion') ~= GMAVersion
        errordlg(['The ' name ' extension is version ' num2str(handle('getVersion')) ...
            '. The current version is ' num2str(GMAVersion) '.'])
        continue
    end
    newHooks = handle('getHooks');
    if isempty(newHooks)
        errordlg(['The ' name ' extension has no hooks, so it is useless.'])
        continue
    end
    oldHooks = hooks;
    for h = 1:length(newHooks)
        if isfield(hooks, newHooks{h})
            hooks.(newHooks{h}){end+1} = name;
        else
            errordlg(['The ' name ' extension is using the ' newHooks{h} ...
                ' hook, which is not implemented in GMA version ' ...
                num2str(GMAVersion) '. ' name ' will not be loaded.'])
            hooks = oldHooks;
            skip = true;
            break
        end
    end
    if skip; continue; end
    % Can't check if dependencies are valid here because not all extensions
    % are loaded yet.
    dependencies{end+1} = handle('getDependencies'); %#ok<AGROW>
    extensionNames{end+1} = name; %#ok<AGROW>
    extensionHandles.(name) = handle;
    waitPosition = waitPosition + waitIncrement;
    waitbar(waitPosition, waithandle)
end

if isempty(hooks.variables)
    close(waithandle)
    errordlg('There are no extensions providing variables.', 'GMA Error')
    return
end
if isempty(hooks.visualizations)
    close(waithandle)
    errordlg('There are no extensions providing visualizations.', 'GMA Error')
    return
end

% FIXME: remove this section because dependencies are done differently
% anyway. Maybe keep the checks.
% Order list of extensions by dependency. Extensions higher up on the list
% are dependencies of those lower on the list.
listLength = length(extensionNames);
sortedMap = zeros(1, listLength);
depOrder = 0;
waitIncrement = 0.4/listLength;
while depOrder < listLength
    found = false;
    for m = find(sortedMap==0)
        if isempty(setdiff(dependencies{m}, extensionNames(sortedMap>0)))
            depOrder = depOrder + 1;
            sortedMap(m) = depOrder;
            found = true;
        end
    end
    if ~found
        close(waithandle)
        % This means that, during the last pass, all of the remaining
        % extensions depended on another extension whose dependencies were
        % not satisfied. This could be caused by two conditions.
        % First check if there is a nonexistent dependency.
        for m = find(sortedMap == 0)
            baddeps = setdiff(dependencies{m}, extensionNames);
            if ~isempty(baddeps)
                errordlg([['The extension ' extensionNames{m} ' depends on ' ...
                    'the following nonexistent extensions:'], baddeps])
                return
            end
        end
        % Otherwise, it's a circular dependency.
        errordlg(['The following extensions have some circular dependency:', extensionNames(sortedMap==0)])
        return
    end
    waitPosition = waitPosition + waitIncrement;
    waitbar(waitPosition, waithandle)
end
% Sanity check
if any(sortedMap == 0)
    close(waithandle)
    errordlg('Internal GMA Error: extension dependency checker ends with unassigned extensions.')
    return
end
% Each entry in sortedMap is the order in which the corresponding extension
% hook should be called.
[~, reorder] = sort(sortedMap);
extensionNames = extensionNames(reorder);
waitbar(0.9, waithandle)
fieldlist = fieldnames(hooks);
waitIncrement = 0.1/length(fieldlist);
waitPosition = 0.9;
for n = 1:length(fieldlist)
    name = fieldlist{n};
    % reorder is the indices of the entries in extensionNames that are also
    % in hooks.(n)
    [~, reorder] = intersect(extensionNames, hooks.(name));
    % This assigns the hooks in the same order as they are in
    % extensionNames.
    hooks.(name) = extensionNames(sort(reorder));
    waitPosition = waitPosition + waitIncrement;
    waitbar(waitPosition, waithandle);
end
close(waithandle)

% This is the skeleton of the GUI.
f = figure('Units', 'pixels', ...
    'Position', [100 100 500 600], 'Toolbar', 'None', ...
    'Tag', 'figure', 'NumberTitle', 'off', ...
    'NextPlot', 'new', 'Name', 'Microstructure Analyzer', ...
    'MenuBar', 'none', 'ResizeFcn', ...
    @(h,e)GMAFunctions.screenManager(guihandles(h),'resize'));
movegui(f, 'center')
uipanel(f, 'Tag', 'Container', 'Units', 'normalized', ...
    'Position', [0 0 1 1], 'Title', '', 'BorderType', 'none');
handles = guihandles(f);
% Get the variables ready.
GMAFunctions.close(handles);
GMAFunctions.set(handles, 'version', GMAVersion)
GMAFunctions.set(handles, 'extensionNames', extensionNames)
GMAFunctions.set(handles, 'extensionHandles', extensionHandles)
GMAFunctions.set(handles, 'hooks', hooks)
% Create the home screen.
GMAFunctions.screenManager(handles, 'switch', 'home')

end % function gma

